use rocket::{
    form::Form,
    get,
    http::{Cookie, CookieJar},
    post,
    request::FlashMessage,
    response::{Flash, Redirect},
    routes, FromForm, Route, State,
};
use rocket_dyn_templates::{tera, Template};
use serde_json::json;
use sqlx::SqlitePool;

use crate::model::user::User;

#[get("/")]
async fn index(flash: Option<FlashMessage<'_>>) -> Template {
    let mut context = tera::Context::new();

    if let Some(msg) = flash {
        context.insert("flash", &msg.into_inner());
    }

    Template::render("auth/login", context.into_json())
}

#[derive(FromForm)]
struct LoginForm {
    name: String,
    password: String,
}

#[post("/", data = "<login>")]
async fn login(
    login: Form<LoginForm>,
    db: &State<SqlitePool>,
    cookies: &CookieJar<'_>,
) -> Flash<Redirect> {
    let user = User::login(db, login.name.clone(), login.password.clone()).await;

    //TODO: be able to use ? for login. This would get rid of the following match clause.
    let user = match user {
        Ok(user) => user,
        Err(_) => {
            return Flash::error(Redirect::to("/auth"), "Falscher Benutzername/Passwort");
        }
    };

    let user_json: String = format!("{}", json!(user));
    cookies.add_private(Cookie::new("loggedin_user", user_json));

    Flash::success(Redirect::to("/"), "Login erfolgreich")
}

#[get("/logout")]
async fn logout(cookies: &CookieJar<'_>, _user: User) -> Flash<Redirect> {
    cookies.remove_private(Cookie::named("loggedin_user"));

    Flash::success(Redirect::to("/auth"), "Logout erfolgreich")
}

pub fn routes() -> Vec<Route> {
    routes![index, login, logout]
}
